<?php
/**
 * Piwik - free/libre analytics platform
 *
 * @link http://piwik.org
 * @license http://www.gnu.org/licenses/gpl-3.0.html GPL v3 or later
 */

namespace Piwik\Plugins\ClickHeat\Reports;

use Piwik\Piwik;
use Piwik\Plugin\Report;
use Piwik\Plugin\ViewDataTable;
use Piwik\Plugins\Actions\Columns\ExitPageUrl;

use Piwik\View;

/**
 * This class defines a new report.
 *
 * See {@link http://developer.piwik.org/api-reference/Piwik/Plugin/Report} for more information.
 */
class GetClickHeatmap extends Base
{
  protected function init()
  {
    parent::init();

    $this->name          = Piwik::translate('ClickHeat_ClickHeatmap');
    $this->dimension     = null;
    $this->documentation = Piwik::translate('');

    // This defines in which order your report appears in the mobile app, in the menu and in the list of widgets
    $this->order = 41;

    // By default standard metrics are defined but you can customize them by defining an array of metric names
    // $this->metrics       = array('nb_visits', 'nb_hits');

    // Uncomment the next line if your report does not contain any processed metrics, otherwise default
    // processed metrics will be assigned
    // $this->processedMetrics = array();

    // Uncomment the next line if your report defines goal metrics
    // $this->hasGoalMetrics = true;

    // Uncomment the next line if your report should be able to load subtables. You can define any action here
    // $this->actionToLoadSubTables = $this->action;

    // Uncomment the next line if your report always returns a constant count of rows, for instance always
    // 24 rows for 1-24hours
    // $this->constantRowsCount = true;

    // If a subcategory is specified, the report will be displayed in the menu under this menu item
    $this->subcategoryId = 'ClickHeat_ClickHeatmap';
  }

  /**
   * Here you can configure how your report should be displayed. For instance whether your report supports a search
   * etc. You can also change the default request config. For instance change how many rows are displayed by default.
   *
   * @param ViewDataTable $view
   */
  public function configureView(ViewDataTable $view)
  {
    if (!empty($this->dimension)) {
      $view->config->addTranslations(array('label' => $this->dimension->getName()));
    }

    // $view->config->show_search = false;
    // $view->requestConfig->filter_sort_column = 'nb_visits';
    // $view->requestConfig->filter_limit = 10';

    $view->config->columns_to_display = array_merge(array('label'), $this->metrics);
  }

  /**
   * Here you can define related reports that will be shown below the reports. Just return an array of related
   * report instances if there are any.
   *
   * @return \Piwik\Plugin\Report[]
   */
  public function getRelatedReports()
  {
    return array(); // eg return array(new XyzReport());
  }

  /**
   * Customized render.
   * @return string
   */
   public function render()
   {
     $view = new View('@ClickHeat/view');
     $view->clickheat_path = '/plugins/ClickHeat/libs/';
     $view->clickheat_menu = array();
     $view->clickheat_loading = '';
     $view->clickheat_cleaner = '';
     $view->clickheat_jsokay = '';
     $view->clickheat_admincookie = '';
     $view->clickheat_iframes = '';
     $view->clickheat_flashes = '';
     $view->clickheat_host = '';
     $view->clickheat_range = '';
     $view->clickheat_alpha = 30;
     $view->clickheat_force_heatmap = '';
     $view->clickheat_browsers = [];
     $view->clickheat_screens = '';
     $view->clickheat_index = '';

     return $view->render();
    }
}
