/**
 ClickHeat : Suivi et analyse des clics / Tracking and clicks analysis
 */

function ClickHeat () {
  if (typeof window.Piwik !== 'object') {
    return;
  }
  /* Main variables */
  this.clickHeatLastIframe = -1;
  this.clickHeatTime = 0;
  this.clickHeatQuota = -1;

  /**
   * <html> tag copy
   * @var {Document}
   */
  this.clickHeatDocument = '';
  this.clickHeatDebug = (document.location.href.indexOf('debugclickheat') !== -1);

  this.init = function () {
    /* Add onmousedown event using listeners */
    addEvtListener(document, 'mousedown', this.processClick);

    /**
     * Add onfocus event on iframes (mostly ads) - Does NOT work with Gecko-powered browsers,
     * because onfocus doesn't exist on iframes
     */
    var iFrames = document.getElementsByTagName('iframe');
    for (var i = 0; i < iFrames.length; i += 1) {
      addEvtListener(iFrames[i], 'focus', this.processClick);
    }

    /* Preparing main variables */
    this.clickHeatDocument = document.documentElement;
  };

  /**
   * Sends data in AJAX POST or plain GET request
   */
  this.sendData = function (params) {
    var clickHeat = window.Piwik.ClickHeat;
    var clickHeatServer = window.Piwik.getTracker().getPiwikUrl()+"/plugins/ClickHeat/lib/index.php";
    /**
     * AJAX request.
     * @var {XMLHttpRequest}
     */
    var xmlhttp;
    try {
      xmlhttp = new ActiveXObject('Msxml2.XMLHTTP');
    } catch (er) {
      try {
        xmlhttp = new ActiveXObject('Microsoft.XMLHTTP');
      } catch (oc) {
        xmlhttp = null;
      }
    }
    if (!xmlhttp && typeof (XMLHttpRequest) !== 'undefined') {
      xmlhttp = new XMLHttpRequest();
    }
    if (xmlhttp) {
      xmlhttp.open('GET', clickHeatServer, true);
      xmlhttp.send(JSON.stringify(params));
      xmlhttp.onreadystatechange = function () {
        if (xmlhttp.readyState === 4 && xmlhttp.status !== 200) {
          var clickHeatImg = new Image();
          clickHeatImg.src = clickHeatServer + '?' + serializeObjectToQuery(params);
        }
      }
    } else {
      var clickHeatImg = new Image();
      clickHeatImg.src = clickHeatServer + '?' + serializeObjectToQuery(params);
    }
  };

  /**
   * Click handler.
   *
   * @param e {event}
   */
  this.processClick = function (e) {
    var c;
    var element;
    /**
     * Mouse horizontal position, in pixels, sent to Matomo.
     * @var {Number} integer
     */
    var x;
    /**
     * Mouse vertical position, in pixels, sent to Matomo.
     * @var {Number} integer
     */
    var y;
    /**
     * Window width in pixels, sent to Matomo
     * @var {Number} integer
     */
    var w;
    /**
     * Window height in pixels, sent to Matomo
     * @var {Number} integer
     */
    var h;
    /**
     * Max possible window width
     * @var {Number}
     */
    var winw;
    /**
     * Max possible window height
     * @var {Number}
     */
    var winh;
    var scrollx;
    var scrolly;
    var clickTime;
    var params;
    var clickHeat = window.Piwik.ClickHeat;
    /* Use a try{} to avoid showing errors to users */
    try {
      if (clickHeat.clickHeatQuota === 0) {
        if (clickHeat.clickHeatDebug) {
          console.debug('Click not logged: quota reached');
        }
        return true;
      }
      /* Look for the real event */
      if (!e) {
        e = window.event;
      }
      c = e.which || e.button;
      element = e.srcElement || null;
      if (c === 0) {
        if (clickHeat.clickHeatDebug) {
          console.debug('Click not logged: no button pressed');
        }
        return true;
      }
      /* Filter for same iframe (focus on iframe => popup ad => close ad => new focus on same iframe) */
      if (element !== null && element.tagName.toLowerCase() === 'iframe') {
        if (element.sourceIndex === clickHeat.clickHeatLastIframe) {
          if (clickHeat.clickHeatDebug) {
            console.debug('Click not logged: same iframe (a click on iframe opens a popup and popup is closed => iframe gets the focus again)');
          }
          return true;
        }
        window.Piwik.ClickHeat.clickHeatLastIframe = element.sourceIndex;
      } else {
        window.Piwik.ClickHeat.clickHeatLastIframe = -1;
      }
      x = e.clientX;
      y = e.clientY;
      w = clickHeat.clickHeatDocument.clientWidth || window.innerWidth;
      h = clickHeat.clickHeatDocument.clientHeight || window.innerHeight;
      scrollx = window.pageXOffset || clickHeat.clickHeatDocument.scrollLeft;
      scrolly = window.pageYOffset || clickHeat.clickHeatDocument.scrollTop;
      winw = Math.max(clickHeat.clickHeatDocument.scrollWidth, clickHeat.clickHeatDocument.offsetWidth, w);
      winh = Math.max(clickHeat.clickHeatDocument.scrollHeight, clickHeat.clickHeatDocument.offsetHeight, h);

      /* Is the click in the viewing area? Not on scrollbars. The problem still exists for FF on the horizontal scrollbar */
      if (x > w || y > h) {
        if (clickHeat.clickHeatDebug) {
          console.debug('Click not logged: out of document (should be a click on scrollbars)');
        }
        return true;
      }
      x += scrollx;
      y += scrolly;
      /* Is the click in the document area? */
      if (x < 0 || y < 0 || x > winw || y > winh) {
        if (clickHeat.clickHeatDebug) {
          console.debug('Click not logged: out of document (should be a click out of the document\'s body)');
        }
        return true;
      }
      /* Check if last click was at least 1 second ago */
      clickTime = new Date();
      if (clickTime.getTime() - clickHeat.clickHeatTime < 1000) {
        if (clickHeat.clickHeatDebug) {
          console.debug('Click not logged: at least 1 second between clicks');
        }
        return true;
      }
      window.Piwik.ClickHeat.clickHeatTime = clickTime.getTime();
      if (clickHeat.clickHeatQuota > 0) {
        window.Piwik.ClickHeat.clickHeatQuota = clickHeat.clickHeatQuota - 1;
      }
      params = {
        'x': x,
        'y': y,
        'w': w,
        'c': c,
        'random': Date()
      };
      if (clickHeat.clickHeatDebug) {
        console.debug('Ready to send click data...');
      }
      clickHeat.sendData(params);
    } catch (err) {
      console.error('An error occurred while processing click (Javascript error): ' + err.message);
    }
    return true;
  }
}

if (typeof window.Piwik === 'object') {
  window.Piwik.ClickHeat = new ClickHeat();
  window.Piwik.ClickHeat.init();
}
